
#include "mmc_config.h"
#include "mmc.h"
#include "spi.h"
#include "delay.h"
#include "main.h"
#include "lcd.h"


// Definitions for MMC/SDC command 
#define CMD0	(0)			// GO_IDLE_STATE 
#define CMD1	(1)			// SEND_OP_COND (MMC) 
#define	ACMD41	(41)		// SEND_OP_COND (SDC)
#define CMD8	(8)			// SEND_IF_COND 
#define CMD9	(9)			// SEND_CSD 
#define CMD10	(10)		// SEND_CID 
#define CMD12	(12)		// STOP_TRANSMISSION 
#define ACMD13	(0x80+13)	// SD_STATUS (SDC) 
#define CMD16	(16)		// SET_BLOCKLEN 
#define CMD17	(17)		// READ_SINGLE_BLOCK 
#define CMD18	(18)		// READ_MULTIPLE_BLOCK 
#define CMD23	(23)		// SET_BLOCK_COUNT (MMC) 
#define	ACMD23	(0x80+23)	// SET_WR_BLK_ERASE_COUNT (SDC) 
#define CMD24	(24)		// WRITE_BLOCK 
#define CMD25	(25)		// WRITE_MULTIPLE_BLOCK 
#define CMD55	(55)		// APP_CMD 
#define CMD58	(58)		// READ_OCR 

// Card type flags (CardType) 
#define CT_MMC		0x01			// MMC ver 3 
#define CT_SD1		0x02			// SD ver 1 
#define CT_SD2		0x04			// SD ver 2 
#define CT_SDC		(CT_SD1|CT_SD2)	// SD 
#define CT_BLOCK	0x08			// Block addressing 

// **********************************************************************************************************************************
// funktionsprototypen von funktionen die nur in dieser datei benutzt werden !

static uint8_t 	mmc_enable(void);
static void 	mmc_disable(void); 
static uint8_t 	mmc_wait_ready (void);
static uint8_t 	mmc_send_cmd (	uint8_t cmd,	uint32_t arg);

#define MMC_CS		GPIO_Pin_3
#define MMC_CS_LOW 	GPIOD->BSRRH = MMC_CS  ;//Reset ON		// Set pin B2 to 0
#define MMC_CS_HIGH	GPIOD->BSRRL = MMC_CS  ;//Reset ON		// Set pin B2 to 1


#define TIMING 1000





// **********************************************************************************************************************************
uint8_t mmc_init (void){

	uint8_t cmd, ty, ocr[4];
	uint16_t n, j;
	unsigned int TimingDelay;

	spi_set_baud (SPI_BaudRatePrescaler_64);
	mmc_disable();

	for (n = 100; n; n--) spi_read_byte();    					// 80+ dummy clocks
	ty = 0;

	if (mmc_send_cmd(CMD0, 0) == 1) 
	{			/* Put the card SPI/Idle state */
		
							/* Initialization timeout = 1 sec */
		if (mmc_send_cmd(CMD8, 0x1AA) == 1) 
		{	/* SDv2? */
			for (n = 0; n < 4; n++) ocr[n] = spi_write_byte(0xFF);	/* Get 32 bit return value of R7 resp */
			if (ocr[2] == 0x01 && ocr[3] == 0xAA) 
			{		
						/* Is the card supports vcc of 2.7-3.6V? */
				TimingDelay = 1000;
				while (TimingDelay-- && mmc_send_cmd(ACMD41, 1UL << 30))
				{
					us_delay(1000);
				}	
				if (TimingDelay && mmc_send_cmd(CMD58, 0) == 0) 
				{		
					
					for (n = 0; n < 4; n++) ocr[n] = spi_write_byte(0xFF);
					ty = (ocr[0] & 0x40) ? CT_SD2 | CT_BLOCK : CT_SD2;	/* Card id SDv2 */
				}
			}
		} else {	/* Not SDv2 card */
			if (mmc_send_cmd(ACMD41, 0) <= 1) 	{	/* SDv1 or MMC? */
				ty = CT_SD1; cmd = ACMD41;	/* SDv1 (ACMD41(0)) */
			} else {
				ty = CT_MMC; cmd = CMD1;	/* MMCv3 (CMD1(0)) */
			}
			TimingDelay = 1000;
			while (TimingDelay-- && mmc_send_cmd(cmd, 0))
			{
				us_delay(1000);
			}
			if (!TimingDelay || mmc_send_cmd(CMD16, 512) != 0)	/* Set block length: 512 */
				ty = 0;
		}
	}

	fat.card_type = ty;
	mmc_disable();
	spi_set_baud (SPI_BaudRatePrescaler_2);

	if( fat.card_type == 0 ){
		return FALSE;
	}
	

	return TRUE;
}

// **********************************************************************************************************************************
static uint8_t mmc_send_cmd (	uint8_t cmd,	uint32_t arg){
	
	uint8_t n, res;
	
	if (cmd & 0x80) {	
		cmd &= 0x7F;
		res = mmc_send_cmd(CMD55, 0);
		if (res > 1) return res;
	}
	

	mmc_disable();
	if ( FALSE == mmc_enable() ){
		return 0xFF;
	}
	// Send command packet 
	spi_write_byte(0x40 | cmd);						// Start + Command index 
	spi_write_byte( (uint8_t)(arg >> 24) );	// Argument[31..24]
	spi_write_byte( (uint8_t)(arg >> 16) );	// Argument[23..16]
	spi_write_byte( (uint8_t)(arg >> 8) );	// Argument[15..8]
	spi_write_byte( (uint8_t)arg );			// Argument[7..0]
	n = 0x01;										// Dummy CRC + Stop 
	if (cmd == CMD0) n = 0x95;						// Valid CRC for CMD0(0) 
	if (cmd == CMD8) n = 0x87;						// Valid CRC for CMD8(0x1AA) 
	spi_write_byte(n);

	// Receive command response 
	if (cmd == CMD12) spi_write_byte(0xFF);				// Skip a stuff byte when stop reading 
	n = 10;											// Wait for a valid response in timeout of 10 attempts 
	do
		res = spi_read_byte();
	while ( (res & 0x80) && --n );

	return res;										// Return with the response value 
}


// **********************************************************************************************************************************
static uint8_t mmc_wait_ready (void){

	unsigned int TimingDelay;
	TimingDelay = 500;

	do{
		if(spi_write_byte(0xFF) == 0xFF ) return TRUE;
		us_delay(1000);
		
	}while ( TimingDelay--);

	return FALSE;
}



// **********************************************************************************************************************************
static uint8_t mmc_enable(){
      
	MMC_CS_LOW;
	spi_write_byte(0xFF);
	if( !mmc_wait_ready() ){
		mmc_disable();
		return FALSE;
	}
	return TRUE;
}

// **********************************************************************************************************************************
static void mmc_disable(){

	MMC_CS_HIGH;   
	spi_write_byte(0xFF);
}









// **********************************************************************************************************************************
// Routine zum schreiben eines Blocks(512Byte) auf die MMC/SD-Karte
// **********************************************************************************************************************************
uint8_t mmc_write_sector (uint32_t addr,uint8_t *buffer){

	uint8_t resp;
	uint8_t retrys;
	uint16_t count;
   	
	if ( !(fat.card_type & CT_BLOCK) ){
		addr *= 512;				// Convert to byte address if needed 
	}
	
	if ( mmc_send_cmd(CMD24, addr) != 0){ 	// enables card		
		return FALSE;
	}

	if ( FALSE == mmc_wait_ready() ){		
		return FALSE;
	}

	spi_write_byte(0xFE);			// Xmit data token 
	
	count = 512;
	do {							// Xmit the 512 byte data block to MMC 
		spi_write_byte(*buffer++);		
	} while (--count);
	
	spi_write_byte(0xFF);			// CRC (Dummy) 
	spi_write_byte(0xFF);
	
	retrys = 20;			
	do{
		resp = spi_read_byte();		// Reveive data response, 20 retrys if not acepted
	}while( (resp & 0x1F) != 0x05 && --retrys);
	
	if ( retrys == 0){				// If not accepted, return with error 		
		return FALSE;
	}
	
	mmc_disable();

	return TRUE;
}


// **********************************************************************************************************************************
// Routine zum lesen eines Blocks(512Byte) von der MMC/SD-Karte
// **********************************************************************************************************************************
uint8_t mmc_read_sector (uint32_t addr,uint8_t *buffer){

	uint8_t token;
	uint16_t count;
	unsigned int TimingDelay;

	if ( !(fat.card_type & CT_BLOCK) ) addr *= 512;	// Convert to byte address if needed

	if ( mmc_send_cmd(CMD17, addr) != 0 ){
		return FALSE;	
	}

	TimingDelay = 20;
	do {							// Wait for data packet in timeout of 200ms 
		token = spi_read_byte();
		us_delay(1000);
	} while ( (token == 0xFF) && TimingDelay-- );
	
	if(token != 0xFE){
		return FALSE;				// If not valid data token, retutn with error 
	}

	count = 512;
	do {							// Receive the data block into buffer 
		*buffer++ = spi_read_byte();
	} while (--count);

	spi_read_byte();				// Discard CRC 
	spi_read_byte();

	mmc_disable();

	return TRUE;					// Return with success 
}





